unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls;

type
  TfrmMain = class(TForm)
    pnlDate: TPanel;
    Memo1: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    Memo2: TMemo;
    Label3: TLabel;
    btnAdd: TButton;
    btnSave: TButton;
    grpBug: TRadioGroup;
    chkVer: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure chkVerClick(Sender: TObject);
  private
    LogFileName: String;
    procedure GetVersionData(var Buf2: PChar);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

uses
  {$IFNDEF WIN32}VER,{$ENDIF}
  StrLib;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  LogFile: TextFile;
begin
  if ParamCount = 0 then
  begin
    MessageBox(Handle, 'Parameter missing.', 'Error', MB_OK);
    Application.Terminate;
  end;
  pnlDate.Caption := FormatDateTime('dddd, mmmm dd yyyy  hh:mm', Now);
  LogFileName := LowerCase(ChangeFileExt(ParamStr(1), '.log'));
  Label2.Caption := Format(Label2.Caption, [LogFileName]);
  if not FileExists(LogFileName) then
  begin
    AssignFile(LogFile, LogFileName);
    Rewrite(LogFile);
    CloseFile(LogFile);
  end;
  Memo2.Lines.LoadFromFile(LogFileName);
  Memo2.Modified := False;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Memo2.Modified then
    if MessageBox(Handle, 'Logfile changed. Save?', 'Confirm', MB_YESNO) = ID_YES then
      Memo2.Lines.SaveToFile(LogFileName);
end;

procedure TfrmMain.btnSaveClick(Sender: TObject);
begin
  if Memo2.Modified then
  begin
    Memo2.Lines.SaveToFile(LogFileName);
    Memo2.Modified := False;
  end;
  Close;
end;

procedure TfrmMain.chkVerClick(Sender: TObject);
begin
  if chkVer.Checked then
    if not FileExists(ParamStr(1)) then
    begin
      MessageBox(Handle, 'Cannot find EXE file.', 'Error', MB_OK);
      chkVer.Checked := False;
    end;
end;

procedure TfrmMain.btnAddClick(Sender: TObject);
var
  Buf1, Buf2: PChar;
begin
  Buf1 := GetMemo2Buffer(Memo1);
  Buf2 := GetMemo2Buffer(Memo2);

  AddToBuff(Buf2, #13#10#9+pnlDate.Caption+#13#10);
  if chkVer.Checked then
    GetVersionData(Buf2);

  case grpBug.ItemIndex of
    0: AddToBuff(Buf2, #13#10'Bug fixed:'+#13#10#9);
    1: AddToBuff(Buf2, #13#10'Changed version to:'+#13#10#9);
    2: AddToBuff(Buf2, #13#10'Improvements:'+#13#10#9);
    3: AddToBuff(Buf2, #13#10'Other fixes:'+#13#10#9);
  end;
  AddToPChar(Buf2, Buf1);
  AddToBuff(Buf2, #13#10#13#10+'********************************************************************************');

  Memo1.Lines.Clear;
  Memo2.Lines.BeginUpdate;
  Memo2.Lines.Clear;
  Memo2.SetTextBuf(Buf2);
  Memo2.Lines.EndUpdate;
  MyStrDispose(Buf1);
  MyStrDispose(Buf2);
  Memo2.Modified := True;
end;

procedure TfrmMain.GetVersionData(var Buf2: PChar);
{$IFNDEF WIN32}
type
  DWORD = LongInt;
{$ENDIF}
var
  Buffer: Pointer;
  FileName: array [0..255] of char; { Filename of version stamped file }
  VerSize: LongInt;                 { Length of buffer for info }
  PData: PChar;                     { Buffer to place the data structure }
  VersionHandle: DWORD;             { Information for use by GetFileVersionInfo }
  {$IFDEF WIN32} Len: DWORD;        { Length, in bytes, of the version-information value. }
  {$ELSE}        Len: Word;         { Length, in bytes, of the version-information value. }
  {$ENDIF}
begin
  StrPCopy(FileName, ParamStr(1));
  VerSize := GetFileVersionInfoSize(FileName, VersionHandle);
  if VerSize > 0 then
  begin
    AddToBuff(Buf2, #13#10);
    PData := GetClearStr(VerSize+1);
    GetFileVersionInfo(FileName, VersionHandle, VerSize, PData);
    VerQueryValue(PData, '\StringFileInfo\040904E4\FileVersion', Buffer, Len);
    if Len > 0 then
    begin
      AddToBuff(Buf2, 'VersionID:'#9);
      AddToPChar(Buf2, Buffer);
      AddToBuff(Buf2, #13#10);
    end;
    VerQueryValue(PData, '\StringFileInfo\040904E4\InternalName', Buffer, Len);
    if Len > 0 then
    begin
      AddToBuff(Buf2, 'InternalName:'#9);
      AddToPChar(Buf2, PChar(Buffer));
      AddToBuff(Buf2, #13#10);
    end;
    VerQueryValue(PData, '\StringFileInfo\040904E4\LegalCopyright', Buffer, Len);
    if Len > 0 then
    begin
      AddToBuff(Buf2, 'LegalCopyright:'#9);
      AddToPChar(Buf2, PChar(Buffer));
      AddToBuff(Buf2, #13#10);
    end;
    VerQueryValue(PData, '\StringFileInfo\040904E4\OriginalFilename', Buffer, Len);
    if Len > 0 then
    begin
      AddToBuff(Buf2, 'Orig.Filename:'#9);
      AddToPChar(Buf2, PChar(Buffer));
      AddToBuff(Buf2, #13#10);
    end;
    MyStrDispose(PData);
  end;
end;

end.

